@@ -222,7 +222,7 @@ SGC is enabled.  */
 #include <math.h>
 #endif
 
-#undef LITTLE_END
+#undef WORDS_BIGENDIAN
 
 #undef PAGEWIDTH
 
@@ -266,4 +266,8 @@ SGC is enabled.  */
 
 #undef TYPE_BITS
 #undef OBJ_ALIGN
+#undef OBJ_ALIGNMENT
 #undef NEG_CSTACK_ADDRESS
+#undef STATIC_LINKING
+#undef ADDR_NO_RANDOMIZE
+#undef LEADING_UNDERSCORE
