@@ -116,7 +116,7 @@ float object_to_float();
 double object_to_double();
 
 /*  error.c  */
-EXTER object sKerror,sKparse_error;
+EXTER object sKerror,sKparse_error,sKreader_error;;
 EXTER object sKwrong_type_argument;
 EXTER object sKcontrol_error;
 EXTER object sKcatch;
@@ -423,11 +423,13 @@ extern object expand_pathname();
 
 /*  prediate.c  */
 
-int eql(),equal(),eq();
+int eql1(),equal1(),eq();
 
 /*  print.d  */
 EXTER object sKupcase;
 EXTER object sKdowncase;
+EXTER object sKpreserve;
+EXTER object sKinvert;
 EXTER object sKcapitalize;
 EXTER object sKstream;
 EXTER object sKreadably;
@@ -491,6 +493,7 @@ EXTER object READtable;
 EXTER int READdefault_float_format;
 EXTER int READbase;
 EXTER bool READsuppress;
+EXTER bool READeval;
 EXTER object siSsharp_comma;
 EXTER bool escape_flag;
 EXTER object delimiting_char;
@@ -589,14 +592,14 @@ EXTER object sLspecial,sLdeclare;
 EXTER object sSvariable_documentation;
 EXTER object sSfunction_documentation;
 EXTER object sSsetf_function;
-#define setf_fn_form(a_) (type_of(a_)==t_cons && MMcar(a_)==sLsetf &&\
-                          type_of(MMcdr(a_))==t_cons && type_of(MMcadr(a_))==t_symbol &&\
+#define setf_fn_form(a_) (consp(a_) && MMcar(a_)==sLsetf &&\
+                          consp(MMcdr(a_)) && type_of(MMcadr(a_))==t_symbol &&\
                           MMcddr(a_)==Cnil)
 
 /*  typespec.c  */
-EXTER object sLcommon,sLnull,sLcons,sLlist,sLsymbol,sLarray,sLvector,sLbit_vector,sLstring;
+EXTER object sLcommon,sLnull,sLcons,sLlist,siLproper_list,sLsymbol,sLarray,sLvector,sLbit_vector,sLstring;
 EXTER object sLsequence,sLsimple_array,sLsimple_vector,sLsimple_bit_vector,sLsimple_string;
-EXTER object sLcompiled_function,sLpathname,sLcharacter,sLnumber,sLrational,sLfloat,sLstring_char;
+EXTER object sLcompiled_function,sLpathname,sLcharacter,sLnumber,sLrational,sLfloat;
 EXTER object sLinteger,sLratio,sLshort_float,sLstandard_char,sLinterpreted_function;
 
 EXTER object sLchar,sLnon_negative_char,sLnegative_char,sLsigned_char,sLunsigned_char;
@@ -605,11 +608,13 @@ EXTER object sLfixnum,sLnon_negative_fixnum,sLnegative_fixnum,sLsigned_fixnum,sL
 EXTER object sLlfixnum,sLnon_negative_lfixnum,sLnegative_lfixnum;
 EXTER object sLsigned_lfixnum,sLunsigned_lfixnum,sLnegative_bignum,sLnon_negative_bignum,sLbase_char;
 
+EXTER object sLsigned_int,sLnon_negative_int,sLnegative_int,sLunsigned_int;
+
 EXTER object sLseqind,sLrnkind;
 
 EXTER object sLcomplex;
 EXTER object sLsingle_float,sLpackage,sLbignum,sLrandom_state,sLdouble_float,sLstream,sLbit,sLreadtable;
-EXTER object sLlong_float,sLhash_table,sLstructure,sLboolean,sLfile_stream;
+EXTER object sLlong_float,sLhash_table,sLstructure,sLboolean,sLfile_stream,sLinput_stream,sLoutput_stream;
 
 #ifdef ANSI_COMMON_LISP
 /* new ansi types */
@@ -696,3 +701,5 @@ object Ifuncall_n(object,int,...);
 object Ivs_values();
 object Icheck_one_type();
 object fSincorrect_type();
+
+
