@@ -58,16 +58,10 @@
 
 #define signals_pending *signalsPendingPtr
 
-#undef DBEGIN_TY
-#define DBEGIN_TY unsigned int
-extern DBEGIN_TY _dbegin;
-
-#define _stacktop cs_limit
-#define _stackbottom cs_org
-
 /* define if there is no _cleanup,   do here what needs
    to be done before calling unexec
-   */   
+   */
+
 #define CLEANUP_CODE \
   setbuf(stdin,0); \
   setbuf(stdout,0);
@@ -117,8 +111,7 @@ extern DBEGIN_TY _dbegin;
    in winnt our heap starts at DBEGIN
    */
 #define NULL_OR_ON_C_STACK(y) \
-    ((y) == 0 || \
-    ((y) > _stacktop && (y) < _stackbottom))     
+    (((unsigned long) (y)) <= cs_org || ((unsigned long) (y)) > (unsigned long) core_end)
 
 #if defined ( IN_FILE ) || defined ( IN_SOCKETS )
 #  define HAVE_NSOCKET
@@ -200,9 +193,12 @@ extern char *GCLExeName ( void );
    (a_)=GCLExeName();\
 } while(0)
 
-#define INSTALL_SEGMENTATION_CATCHER \
-  	 (void) signal(SIGSEGV,segmentation_catcher)
+/* #define INSTALL_SEGMENTATION_CATCHER \
+   (void) signal(SIGSEGV,segmentation_catcher) */
 
+#define MAXCORE ((char *)((unsigned long)DBEGIN+(unsigned long)(MAXPAGE-1)*PAGESIZE))
+
+#define malloc gcl_malloc
 /* Begin for cmpinclude */
 
 
