@@ -71,7 +71,7 @@ typedef unsigned char  uqfixnum;
 #error Cannot calculate FILL_BITS
 #endif
 
-#ifdef LITTLE_END
+#ifndef WORDS_BIGENDIAN
 
 #define FIRSTWORD fixnum e:1,m:1,f:1,s:1,z:4,t:8,w:FILL_BITS
 #define MARKWORD  fixnum e:1,mf:2,   s:1,z:4,t:8,w:FILL_BITS
@@ -88,6 +88,8 @@ typedef unsigned char  uqfixnum;
 #endif
 
 #define NOT_OBJECT_ALIGNED(a_) fobj(a_)->td.emf
+#define ROUNDUP(x_,y_) (((unsigned long)(x_)+(y_ -1)) & ~(y_ -1))
+#define ROUNDDN(x_,y_) (((unsigned long)(x_)) & ~(y_ -1))
 
 #ifndef PAGEWIDTH
 #define PAGEWIDTH 11
@@ -193,7 +195,7 @@ struct fixnum_struct {
 #define make_free(a_)            {*(fixnum *)(a_)&=TYPE_BITS;(a_)->d.f=1;}
 #define make_unfree(a_)          {(a_)->d.f=0;}
 
-#define is_cons(a_)              (is_imm_fixnum(Zcdr(a_)) || !(a_)->d.e)
+#define valid_cdr(a_)              (!(a_)->d.e || is_imm_fixnum(Zcdr(a_)))
 
 struct shortfloat_struct {
 
@@ -302,11 +304,17 @@ struct symbol {
 
 };
 
-EXTER char CnilCt[3*sizeof(struct symbol)] OBJ_ALIGN;
+/* EXTER char CnilCt[3*sizeof(struct symbol)] OBJ_ALIGN; */
 
-#define Cnil   ((object)(CnilCt))
-#define Ct     ((object)((char *)Cnil+sizeof(struct symbol)))
-#define Dotnil ((object)((char *)Ct+sizeof(struct symbol)))
+/* #define Cnil   ((object)(CnilCt)) */
+/* #define Ct     ((object)((char *)Cnil+sizeof(struct symbol))) */
+/* #define Dotnil ((object)((char *)Ct+sizeof(struct symbol))) */
+
+EXTER struct symbol Cnil_body OBJ_ALIGN;
+EXTER struct symbol Ct_body OBJ_ALIGN;
+
+#define Cnil ((object)&Cnil_body)
+#define Ct   ((object)&Ct_body)
 #define sLnil Cnil
 #define sLt Ct
 
@@ -359,7 +367,8 @@ struct cons {
 enum httest {   /*  hash table key test function  */
   htt_eq,       /*  eq  */
   htt_eql,      /*  eql  */
-  htt_equal     /*  equal  */
+  htt_equal,    /*  equal  */
+  htt_equalp    /*  equalp  */
 };
 
 struct htent {      /*  hash table entry  */
@@ -383,35 +392,34 @@ struct hashtable {           /*  hash table header  */
 
 };
 
-enum aelttype {   /*  array element type  */
- aet_object,      /*  t  */
- aet_ch,          /*  string-char  */
- aet_bit,         /*  bit  */
- aet_fix,         /*  fixnum  */
- aet_nnfix,       /*  non-neg fixnum  */
- aet_sf,          /*  short-float  */
- aet_lf,          /*  plong-float  */
- aet_char,        /* signed char */
- aet_nnchar,      /* non-neg char */
- aet_uchar,       /* unsigned char */
- aet_short,       /* signed short */
- aet_nnshort,     /*  non-neg short   */
- aet_ushort,      /*  unsigned short   */
- aet_last
-};
+typedef struct {
+  void *dflt;
+  object *namep;
+  unsigned char size;
+} aet_type_struct;
+
+
+#if SIZEOF_LONG == 4
+#define HFILL_BITS 15
+#elif SIZEOF_LONG == 8
+#define HFILL_BITS 31
+#else
+#error Cannot calculate HFILL_BITS
+#endif
 
 struct array {           /*  array header  */
 
   FIRSTWORD;
 
   object  a_displaced;   /*  displaced  */
-  hfixnum a_rank;        /*  array rank  */
+  hfixnum a_hasfillp:1;  /*  fillp compatability */
+  hfixnum a_rank:HFILL_BITS;        /*  array rank  */
   hfixnum a_elttype;     /*  element type  */
   object *a_self;        /*  pointer to the array  */
+  fixnum *a_dims;        /*  table of dimensions  */
+  fixnum  a_dim;         /*  dimension  */
   hfixnum a_adjustable;  /*  adjustable flag  */
   hfixnum a_offset;      /*  bitvector offset  */
-  fixnum  a_dim;         /*  dimension  */
-  fixnum *a_dims;        /*  table of dimensions  */
   SPAD;
 
 };
@@ -423,7 +431,8 @@ struct vector {           /*  vector header  */
   FIRSTWORD;
 
   object  v_displaced;    /*  displaced  */
-  hfixnum v_hasfillp;     /*  has-fill-pointer flag  */
+  hfixnum v_hasfillp:1;   /*  has-fill-pointer flag  */
+  hfixnum v_unused:HFILL_BITS;   /*  rank compatibility  */
   hfixnum v_elttype;      /*  element type  */
   object *v_self;         /*  pointer to the vector  */
   fixnum  v_fillp;        /*  fill pointer  */
@@ -441,7 +450,8 @@ struct string {           /*  string header  */
   FIRSTWORD;
 
   object  st_displaced;    /*  displaced  */
-  hfixnum st_hasfillp;     /*  has-fill-pointer flag  */
+  hfixnum st_hasfillp:1;     /*  has-fill-pointer flag  */
+  hfixnum st_unused:HFILL_BITS;   /*  rank compatibility  */
   hfixnum st_adjustable;   /*  adjustable flag  */
   char    *st_self;        /*  pointer to the string  */
   fixnum   st_fillp;       /*  fill pointer  */
@@ -454,21 +464,46 @@ struct string {           /*  string header  */
 #define stack_string(a_,b_) struct string _s={0};\
                             object a_=(object)&_s;\
                             set_type_of((a_),t_string);\
-                            (a_)->st.st_self=(b_);\
-                            (a_)->st.st_dim=(a_)->st.st_fillp=strlen(b_);
+                            (a_)->st.st_self=(void *)(b_);\
+                            (a_)->st.st_dim=(a_)->st.st_fillp=strlen(b_)
 
-#define TYPE_ERROR(a_,b_) {stack_string(tp_err,"~S is not of type ~S.");\
-                           Icall_error_handler(sKwrong_type_argument,tp_err,2,(a_),(b_));}
+#define stack_fixnum(a_,b_) struct fixnum_struct _s={0};\
+                            object a_;\
+                            if (is_imm_fix(b_)) (a_)=make_fixnum(b_); else {\
+                            (a_)=(object)&_s;\
+                            set_type_of((a_),t_fixnum);\
+                            (a_)->FIX.FIXVAL=(b_);}
 
-#define CONTROL_ERROR(a_) {stack_string(tp_err,#a_);\
-                           Icall_error_handler(sKcontrol_error,tp_err,0);}
+/*FIXME the stack stuff is dangerous It works for error handling, but
+  simple errors may evan pass the format tring up the stack as a slot
+  in ansi*/
+/* #define TYPE_ERROR(a_,b_) {stack_string(tp_err,"~S is not of type ~S.");\ */
+/*                            Icall_error_handler(sKwrong_type_argument,tp_err,2,(a_),(b_));} */
+
+#define TYPE_ERROR(a_,b_) ({object tp_err=make_simple_string("~S is not of type ~S.");\
+                           Icall_error_handler(sKwrong_type_argument,tp_err,2,(a_),(b_));})
+
+#define CONTROL_ERROR(a_) ({object tp_err=make_simple_string(a_);\
+                           Icall_error_handler(sKcontrol_error,tp_err,0);})
+
+#define READER_ERROR(a_,b_)  ({object rd_err=make_simple_string("Read error on stream ~S: " b_);\
+                              Icall_error_handler(sKreader_error,rd_err,1,(a_));})
+
+#define NERROR(a_)  ({object fmt=make_simple_string(a_ ": line ~a, file ~a, function ~a");\
+                    {object line=make_fixnum(__LINE__);\
+                    {object file=make_simple_string(__FILE__);\
+                    {object function=make_simple_string(__FUNCTION__);\
+                     Icall_error_handler(sKerror,fmt,3,line,file,function);}}}})
+
+#define ASSERT(a_) if (!(a_)) NERROR("The assertion " #a_ " failed")
 
 struct ustring {
 
   FIRSTWORD;
 
   object         ust_displaced;
-  hfixnum        ust_hasfillp;
+  hfixnum        ust_hasfillp:1;
+  hfixnum        ust_unused:HFILL_BITS;   /*  rank compatibility  */
   hfixnum        ust_adjustable;  
   unsigned char *ust_self;
   fixnum         ust_fillp;
@@ -480,6 +515,9 @@ struct ustring {
 #define USHORT_GCL(x,i)  (((unsigned short *)(x)->ust.ust_self)[i])
 #define  SHORT_GCL(x,i)  ((( short *)(x)->ust.ust_self)[i])
 
+#define UINT_GCL(x,i)  (((unsigned int *)(x)->ust.ust_self)[i])
+#define  INT_GCL(x,i)  ((( int *)(x)->ust.ust_self)[i])
+
 #define BV_OFFSET(x) ((type_of(x)==t_bitvector ? x->bv.bv_offset : \
                        type_of(x)== t_array ? x->a.a_offset : (abort(),0)))
 
@@ -491,7 +529,8 @@ struct bitvector {         /*  bitvector header  */
   FIRSTWORD;
 
   object   bv_displaced;   /*  displaced  */
-  hfixnum  bv_hasfillp;    /*  has-fill-pointer flag  */
+  hfixnum  bv_hasfillp:1;  /*  has-fill-pointer flag  */
+  hfixnum  bv_unused:HFILL_BITS;   /*  rank compatibility  */
   hfixnum  bv_elttype;     /*  not used  */
   char    *bv_self;        /*  pointer to the bitvector  */
   fixnum   bv_fillp;       /*  fill pointer  */
@@ -515,10 +554,10 @@ struct fixarray {            /*  fixnum array header  */
   hfixnum   fixa_rank;       /*  array rank  */
   hfixnum   fixa_elttype;    /*  element type  */
   fixnum   *fixa_self;       /*  pointer to the array  */
+  fixnum   *fixa_dims;       /*  table of dimensions  */
+  fixnum    fixa_dim;        /*  dimension  */
   hfixnum   fixa_adjustable; /*  adjustable flag  */
   hfixnum   fixa_offset;     /*  not used  */
-  fixnum    fixa_dim;        /*  dimension  */
-  fixnum   *fixa_dims;       /*  table of dimensions  */
   SPAD;
   
 };
@@ -531,10 +570,10 @@ struct sfarray {                  /*  short-float array header  */
   hfixnum      sfa_rank;          /*  array rank  */
   hfixnum      sfa_elttype;       /*  element type  */
   shortfloat  *sfa_self;          /*  pointer to the array  */
+  fixnum      *sfa_dims;          /*  table of dimensions  */
+  fixnum       sfa_dim;           /*  dimension  */
   hfixnum      sfa_adjustable;    /*  adjustable flag  */
   hfixnum      sfa_offset;        /*  not used  */
-  fixnum       sfa_dim;           /*  dimension  */
-  fixnum      *sfa_dims;          /*  table of dimensions  */
   SPAD;
 
 
@@ -549,10 +588,10 @@ struct lfarray {             /*  plong-float array header  */
   hfixnum     lfa_rank;      /*  array rank  */
   hfixnum     lfa_elttype;   /*  element type  */
   longfloat  *lfa_self;      /*  pointer to the array  */
+  fixnum     *lfa_dims;      /*  table of dimensions  */
+  fixnum      lfa_dim;       /*  dimension  */
   hfixnum     lfa_adjustable;/*  adjustable flag  */
   hfixnum     lfa_offset;    /*  not used  */
-  fixnum      lfa_dim;       /*  dimension  */
-  fixnum     *lfa_dims;      /*  table of dimensions  */
   SPAD;
 
 };
@@ -702,8 +741,21 @@ enum chattrib {       /*  character attribute  */
  cat_constituent      /*  constituent  */
 };
 
+enum chatrait {       /*  character attribute  */
+ trait_alpha,         /*  alphabetic  */
+ trait_digit,         /*  digits      */
+ trait_alphadigit,    /*  alpha/digit */
+ trait_package,       /*  package mrk */
+ trait_plus,          /*  plus sign   */
+ trait_minus,         /*  minus sign  */
+ trait_ratio,         /*  ratio mrk   */
+ trait_exp,           /*  expon mrk   */
+ trait_invalid        /*  unreadable  */
+};
+
 struct rtent {               /*  read table entry  */
  enum chattrib rte_chattrib; /*  character attribute  */
+ enum chatrait rte_chatrait; /*  constituent trait */
  object        rte_macro;    /*  macro function  */
  object        *rte_dtab;    /*  pointer to the  */
                              /*  dispatch table  */
@@ -718,6 +770,8 @@ struct readtable {       /*  read table  */
   FIRSTWORD;
 
   struct rtent *rt_self; /*  read table itself  */
+  object rt_case;
+  SPAD;
 
 };
 
@@ -902,12 +956,20 @@ struct freelist {
 /*
  Type_of.
 */
-#define type_of(x)       ({object _z=(object)(x);\
-                           is_imm_fixnum(_z) ? t_fixnum : \
-                           (_z==Cnil || _z==Ct || _z==Dotnil ? t_symbol : \
-                           (is_cons(_z) ?  t_cons  : _z->d.t));})
+#define type_of(x)       ({register object _z=(object)(x);\
+                           _z==Cnil ? t_symbol : \
+                           (is_imm_fixnum(_z) ? t_fixnum : \
+                           (valid_cdr(_z) ?  t_cons  : _z->d.t));})
 #define set_type_of(x,y) ({object _x=(object)(x);enum type _y=(y);*(fixnum *)(_x)&=TYPE_BITS;\
                            if (_y!=t_cons) {_x->d.e=1;_x->d.t=_y;}})
+
+#define consp(x)         ({register object _z=(object)(x);\
+                           (_z!=Cnil && !is_imm_fixnum(_z) && valid_cdr(_z));})
+#define listp(x)         ({register object _z=(object)(x);\
+                           (!is_imm_fixnum(_z) && valid_cdr(_z));})
+#define atom(x)          ({register object _z=(object)(x);\
+                           (_z==Cnil || is_imm_fixnum(_z) || !valid_cdr(_z));})
+
 /*
  Storage manager for each type.
 */
@@ -1031,7 +1093,7 @@ EXTER char *tmp_alloc;
 /*  For IEEEFLOAT, the double may have exponent in the second word
 (little endian) or first word.*/
 
-#if defined(I386) || defined(LITTLE_END)
+#if !defined(WORDS_BIGENDIAN)
 #define HIND 1  /* (int) of double where the exponent and most signif is */
 #define LIND 0  /* low part of a double */
 #else /* big endian */
@@ -1214,31 +1276,102 @@ EXTER unsigned plong signals_allowed, signals_pending  ;
    initialized to be placed in .data as opposed to .bss and thereby
    initialized by the loader.  20050616 CM*/
 
-#define endp(x) ({\
-    static struct cons s_my_dot={Dotnil,Dotnil};\
-    object _x=(x);\
-    bool _b=FALSE;\
-    \
-    if (type_of(_x)==t_cons) {\
-       if (type_of(_x->c.c_cdr)!=t_cons && _x->c.c_cdr!=Cnil)\
-          s_my_dot.c_car=_x->c.c_cdr;\
-       else \
-          s_my_dot.c_car=Dotnil;\
-    } else {\
-       if (_x==s_my_dot.c_car)\
-          x=(object)&s_my_dot;\
-       else {\
-         s_my_dot.c_car=Dotnil;\
-         if (_x==Cnil || _x==Dotnil)\
-             _b=TRUE;\
-         else\
-             FEwrong_type_argument(sLlist, _x);\
-       }\
-    }\
-    _b;\
-    })
+/* #define endp(x) ({\ */
+/*     static struct cons s_my_dot={Dotnil,Dotnil};\ */
+/*     object _x=(x);\ */
+/*     bool _b=FALSE;\ */
+/*     \ */
+/*     if (type_of(_x)==t_cons) {\ */
+/*        if (type_of(_x->c.c_cdr)!=t_cons && _x->c.c_cdr!=Cnil)\ */
+/*           s_my_dot.c_car=_x->c.c_cdr;\ */
+/*        else \ */
+/*           s_my_dot.c_car=Dotnil;\ */
+/*     } else {\ */
+/*        if (_x==s_my_dot.c_car)\ */
+/*           x=(object)&s_my_dot;\ */
+/*        else {\ */
+/*          s_my_dot.c_car=Dotnil;\ */
+/*          if (_x==Cnil || _x==Dotnil)\ */
+/*              _b=TRUE;\ */
+/*          else\ */
+/*              FEwrong_type_argument(sLlist, _x);\ */
+/*        }\ */
+/*     }\ */
+/*     _b;\ */
+/*     }) */
 
-#define endp_prop(a) (type_of(a)==t_cons ? FALSE : ((a)==Cnil ? TRUE : (FEwrong_type_argument(sLlist, (a)),FALSE)))
+/* #define endp_prop(a) (type_of(a)==t_cons ? FALSE : ((a)==Cnil ? TRUE : (FEwrong_type_argument(sLlist, (a)),FALSE))) */
     
-#define proper_list(a) (type_of(a)==t_cons || (a)==Cnil)
-#define fix_dot(a) ((a) == Dotnil ? Cnil : (type_of(a)==t_cons && (a)->c.c_cdr==Dotnil ? (a)->c.c_car : (a))) 
+/* #define proper_list(a) (type_of(a)==t_cons || (a)==Cnil) */
+/* #define fix_dot(a) ((a) == Dotnil ? Cnil : (type_of(a)==t_cons && (a)->c.c_cdr==Dotnil ? (a)->c.c_car : (a)))  */
+
+#define endp(a) (consp(a) ? FALSE : ((a)==Cnil ? TRUE : ({TYPE_ERROR((a),sLlist);FALSE;})))
+
+
+#define eql(a_,b_)    ({register object _a=(a_);register object _b=(b_);_a==_b || eql1(_a,_b);})
+#define equal(a_,b_)  ({register object _a=(a_);register object _b=(b_);_a==_b || equal1(_a,_b);})
+#define equalp(a_,b_) ({register object _a=(a_);register object _b=(b_);_a==_b || equalp1(_a,_b);})
+
+extern void *stack_alloc_start,*stack_alloc_end;
+
+
+
+#define stack_alloc_on(n_) ({void *_v=alloca(n_*PAGESIZE+OBJ_ALIGNMENT-1);\
+                             if (_v) {\
+                                stack_alloc_start=(void *)ROUNDUP(_v,OBJ_ALIGNMENT);\
+                                memset(_v,0,stack_alloc_start-_v);\
+                                _v+=n_*PAGESIZE+OBJ_ALIGNMENT-1;\
+                                stack_alloc_end=(void *)ROUNDDN(_v,OBJ_ALIGNMENT);\
+                                memset(stack_alloc_end,0,_v-stack_alloc_end);\
+                             };\
+                           })
+     
+#define stack_alloc_off() ({stack_alloc_start=stack_alloc_end=NULL;})
+            
+#define maybe_alloc_on_stack(n_,t_) ({void *_v=OBJNULL;\
+                                      if (stack_alloc_start) {\
+                                         unsigned _n=ROUNDUP(n_,OBJ_ALIGNMENT);\
+                                         if (stack_alloc_end-stack_alloc_start>_n) {\
+                                           _v=stack_alloc_start;\
+                                           stack_alloc_start+=_n;\
+                                           if (t_>=0) set_type_of(_v,t_);\
+                                         } else stack_alloc_off();\
+                                      }\
+                                      _v;})
+
+
+#define stack_pages_left ({fixnum _val;int _w;\
+                           _val=cs_limit-&_w;\
+                           _val=_val<0 ? -_val : _val;\
+                           _val=(_val>>PAGEWIDTH);})
+
+#define myfork() ({int _p[2],_j=0;pid_t _pid;\
+                   pipe(_p);\
+                   _pid=fork();\
+                   if (!_pid) { \
+                      object _x=sSAchild_stack_allocA->s.s_dbind;\
+                      enum type _tp=type_of(_x);\
+                      float _fac= _tp==t_shortfloat ? sf(_x) : (_tp==t_longfloat ? lf(_x) : 0.8);\
+                      fixnum _n=_fac*stack_pages_left;\
+                      if (_n>0) stack_alloc_on(_n);\
+                      close(0);close(1);close(2);\
+                      _j=1;\
+                   } \
+                   close(_p[1-_j]);\
+		   make_cons(make_fixnum(_pid),make_fixnum(_p[_j]));})
+
+#define make_fd_stream(fd_,mode_,st_,buf_) ({object _x=alloc_object(t_stream);\
+                                            _x->sm.sm_mode=mode_;\
+                                            _x->sm.sm_fp=fdopen(fd_,st_);\
+                                            _x->sm.sm_buffer=buf_;\
+                                            setbuf(_x->sm.sm_fp,_x->sm.sm_buffer);\
+                                            _x->sm.sm_object0=sLcharacter;\
+                                            _x->sm.sm_object1=Cnil;\
+                                            _x->sm.sm_fd=fd_;\
+                                            _x;})
+
+#define writable_ptr(a_) (((void *)(a_)>=DBEGIN && (void *)(a_)<(void *)heap_end) || is_imm_fixnum(a_))
+
+#define write_pointer_object(a_,b_) fSwrite_pointer_object(a_,b_)
+
+#define read_pointer_object(a_) fSread_pointer_object(a_)
