@@ -311,6 +311,8 @@ typedef void (*funcvoid)(void);
 /* makefun.c:113:OF */ extern object fSset_key_struct (object key_struct_ind); /* (key_struct_ind) object key_struct_ind; */
 /* makefun.c:122:OF */ extern void SI_makefun (char *strg, object (*fn) (/* ??? */), unsigned int argd); /* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; */
 /* makefun.c:131:OF */ extern void LISP_makefun (char *strg, object (*fn) (/* ??? */), unsigned int argd); /* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; */
+/* makefun.c:122:OF */ extern void SI_makefunm (char *strg, object (*fn) (/* ??? */), unsigned int argd); /* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; */
+/* makefun.c:131:OF */ extern void LISP_makefunm (char *strg, object (*fn) (/* ??? */), unsigned int argd); /* (strg, fn, argd) char *strg; object (*fn)(); unsigned int argd; */
 /* makefun.c:167:OF */ extern object fSinvoke (object x); /* (x) object x; */
 /* mapfun.c:324:OF */ extern void gcl_init_mapfun (void); /* () */
 /* multival.c:32:OF */ extern void Lvalues (void); /* () */
@@ -435,11 +437,11 @@ typedef void (*funcvoid)(void);
 /* predicate.c:346:OF */ extern object fLcompiled_function_p (object x0); /* (x0) object x0; */
 /* predicate.c:367:OF */ extern object fLcommonp (object x0); /* (x0) object x0; */
 /* predicate.c:379:OF */ extern object fLeq (object x0, object x1); /* (x0, x1) object x0; object x1; */
-/* predicate.c:393:OF */ extern int eql (object x, object y); /* (x, y) object x; object y; */
+/* predicate.c:393:OF */ extern int eql1 (object x, object y); /* (x, y) object x; object y; */
 /* predicate.c:455:OF */ extern object fLeql (object x0, object x1); /* (x0, x1) object x0; object x1; */
-/* predicate.c:469:OF */ extern int equal (register object x, register object y); /* (x, y) register object x; register object y; */
+/* predicate.c:469:OF */ extern int equal1 (register object x, register object y); /* (x, y) register object x; register object y; */
 /* predicate.c:543:OF */ extern object fLequal (object x0, object x1); /* (x0, x1) object x0; object x1; */
-/* predicate.c:557:OF */ extern bool equalp (object x, object y); /* (x, y) object x; object y; */
+/* predicate.c:557:OF */ extern bool equalp1 (object x, object y); /* (x, y) object x; object y; */
 /* predicate.c:681:OF */ extern object fLequalp (object x0, object x1); /* (x0, x1) object x0; object x1; */
 /* predicate.c:750:OF */ extern bool contains_sharp_comma (object x); /* (x) object x; */
 /* predicate.c:797:OF */ extern object fScontains_sharp_comma (object x0); /* (x0) object x0; */
@@ -488,6 +490,7 @@ typedef void (*funcvoid)(void);
 /* typespec.c:191:OF */ extern void check_type_cons (object *p); /* (p) object *p; */
 /* typespec.c:198:OF */ extern void check_type_stream (object *p); /* (p) object *p; */
 /* typespec.c:205:OF */ extern void check_type_readtable (object *p); /* (p) object *p; */
+/* typespec.c:205:OF */ extern void check_type_readtable_no_default (object *p); /* (p) object *p; */
 /* typespec.c:213:OF */ extern void check_type_or_Pathname_string_symbol (object *p); /* (p) object *p; */
 /* typespec.c:225:OF */ extern void check_type_or_pathname_string_symbol_stream (object *p); /* (p) object *p; */
 /* typespec.c:236:OF */ extern void check_type_random_state (object *p); /* (p) object *p; */
@@ -848,9 +851,6 @@ Lpackage_used_by_list(void);
 void
 Lstandard_char_p(void);
 
-void
-Lstring_char_p(void);
-
 void
 Lchar_code(void);
 
@@ -941,9 +941,6 @@ Lreconc(void);
 void
 Lstandard_char_p(void);
 
-void
-Lstring_char_p(void);
-
 void
 Lcharacter(void);
 
@@ -1409,7 +1406,6 @@ void Lforce_output(void);
 void Lnthcdr(void);
 void Llogior(void);
 void Lchar_downcase(void);
-void Lstring_char_p(void);
 void Lstream_element_type(void);
 void Lpackage_used_by_list(void);
 void Ldivide(void);
@@ -1770,6 +1766,9 @@ gcl_isnormal_double(double);
 int 
 gcl_isnormal_float(float);
 
+object
+find_init_name1(char *,unsigned);
+
 void wipe_stack   (VOL void *);
 void clear_c_stack(VOL unsigned);
 
@@ -1817,3 +1816,5 @@ gprof_cleanup(void);
 
 unsigned long
 ihash_equal1(object,int);
+
+/* void call_after_gbc_hook(int); */
