@@ -34,11 +34,10 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 static int
 t_from_type(object);
 
-DEFVAR("*AFTER-GBC-HOOK*",sSAafter_gbc_hookA,SI,sLnil,"");
+/* DEFVAR("*AFTER-GBC-HOOK*",sSAafter_gbc_hookA,SI,sLnil,""); */
 DEFVAR("*IGNORE-MAXIMUM-PAGES*",sSAignore_maximum_pagesA,SI,sLt,"");
 #define IGNORE_MAX_PAGES (sSAignore_maximum_pagesA ==0 || sSAignore_maximum_pagesA->s.s_dbind !=sLnil) 
 
-static void call_after_gbc_hook(int t);
 
 #ifdef DEBUG_SBRK
 int debug;
@@ -85,6 +84,7 @@ struct rlimit data_rlimit;
 int reserve_pages_for_signal_handler =30;
 int hole_overrun=0;
 
+void *stack_alloc_start=NULL,*stack_alloc_end=NULL;
 
 /* If  (n >= 0 ) return pointer to n pages starting at heap end,
    These must come from the hole, so if that is exhausted you have
@@ -95,11 +95,13 @@ int hole_overrun=0;
    If not in_signal_handler then try to keep a minimum of
    reserve_pages_for_signal_handler pages on hand in the hole
  */
+/* static int pending_agbch_call; */
 char *
 alloc_page(long n)
 {
 	char *e;
 	long m;
+
 	e = heap_end;
 	if (n >= 0) {
 		if (n >=
@@ -123,6 +125,7 @@ eg to add 20 more do (si::set-hole-size %ld %d)\n...start over ", new_holepage,
 			hole_overrun=1;
 			GBC(t_relocatable);
 			hole_overrun=0;
+/* 			pending_agbch_call=1; */
 #ifdef SGC
 			if (in_sgc)
 			  {hole_overrun=1;sgc_start();hole_overrun=0;
@@ -198,9 +201,9 @@ add_page_to_freelist(char *p, struct typemanager *tm)
  make_free(x);
 #ifdef SGC
  if (sgc_enabled && tm->tm_sgc)
-   {if (type_of(x)!=t_cons) x->d.s=SGC_RECENT; /*FIXME, can sgc mark cons on 64bit*/
+   {if (!consp(x)) x->d.s=SGC_RECENT; /*FIXME, can sgc mark cons on 64bit*/
     sgc_type_map[np] = (SGC_PAGE_FLAG | SGC_TEMP_WRITABLE);}
- else {if (type_of(x)!=t_cons) x->d.s = SGC_NORMAL;}
+ else {if (!consp(x)) x->d.s = SGC_NORMAL;}
  
  /* array headers must be always writable, since a write to the
     body does not touch the header.   It may be desirable if there
@@ -228,13 +231,14 @@ type_name(int t)
 { return make_simple_string(tm_table[(int)t].tm_name+1);}
 
 
-static void
-call_after_gbc_hook(t)
-{ if (sSAafter_gbc_hookA && sSAafter_gbc_hookA->s.s_dbind!= Cnil)
-    { set_up_string_register(tm_table[(int)t].tm_name+1);
-      ifuncall1(sSAafter_gbc_hookA->s.s_dbind,intern(string_register,system_package));
-    }
-}
+/* void */
+/* call_after_gbc_hook(t) */
+/* { pending_agbch_call=0; */
+/*   if (sSAafter_gbc_hookA && sSAafter_gbc_hookA->s.s_dbind!= Cnil) */
+/*     { set_up_string_register(tm_table[(int)t].tm_name+1); */
+/*       ifuncall1(sSAafter_gbc_hookA->s.s_dbind,intern(string_register,system_package)); */
+/*     } */
+/* } */
 
 /* #define PERCENT_FREE(tm)  ((tm->tm_percent_free ? tm->tm_percent_free : 10)/100.0) */
 #define PERCENT_FREE(tm)  ((tm->tm_percent_free ? tm->tm_percent_free : 30)/100.0)
@@ -355,9 +359,14 @@ alloc_object(enum type t)
 	 int must_have_more_pages;
 	 unsigned long i;
 
-ONCE_MORE:
 	tm = tm_of(t);
 
+	if ((obj=maybe_alloc_on_stack(tm->tm_size,t))) {
+/* 	  printf("alloc_object %u %u %u %p\n",getpid(),t,tm->tm_size,obj); */
+	  return obj;
+	}
+
+ONCE_MORE:
         CHECK_INTERRUPT;	 
 
 	obj = tm->tm_free;
@@ -372,6 +381,7 @@ ONCE_MORE:
 		}
 		p = alloc_page(1);
 		add_page_to_freelist(p,tm);
+/* 		if (pending_agbch_call) call_after_gbc_hook(t); */
 		obj = tm->tm_free;
 		if (tm->tm_npage >= tm->tm_maxpage)
 			goto CALL_GBC;
@@ -405,7 +415,7 @@ CALL_GBC:
 	     tm->tm_adjgbccnt*=(double)j/tm->tm_maxpage;
 	   } 
 	}
-	call_after_gbc_hook(t);
+/* 	call_after_gbc_hook(t); */
 	goto ONCE_MORE;
 
 EXHAUSTED:
@@ -420,7 +430,7 @@ Use ALLOCATE to expand the space.",
 		2, vs_top[-2], vs_top[-1], Cnil, Cnil);
 	vs_popp;
 	vs_popp;
-	call_after_gbc_hook(t);
+/* 	call_after_gbc_hook(t); */
 	goto ONCE_MORE;
 }
 
@@ -433,8 +443,17 @@ make_cons(object a, object d)
 	 int must_have_more_pages;
 	 unsigned long i;
 
+	if ((obj=maybe_alloc_on_stack(tm->tm_size,t_cons))) {
+/* 	  printf("make_cons %u %u %u %p %p %p\n",getpid(),t_cons,tm->tm_size,obj,a,d); */
+	  obj->c.c_car=a;
+	  obj->c.c_cdr=d;
+	  return obj;
+	}
+
+
 ONCE_MORE:
         CHECK_INTERRUPT;
+
 	obj = tm->tm_free;
 	if (obj == OBJNULL) {
 		if (tm->tm_npage >= tm->tm_maxpage)
@@ -447,6 +466,7 @@ ONCE_MORE:
 		}
 		p = alloc_page(1);
 		add_page_to_freelist(p,tm);
+/* 		if (pending_agbch_call) call_after_gbc_hook(t_cons); */
 		obj = tm->tm_free ;
 		if (tm->tm_npage >= tm->tm_maxpage)
 			goto CALL_GBC;
@@ -476,7 +496,7 @@ CALL_GBC:
 	     tm->tm_adjgbccnt*=(double)j/tm->tm_maxpage;
 	   } 
 	}
-	call_after_gbc_hook(t_cons);
+/* 	call_after_gbc_hook(t_cons); */
 	goto ONCE_MORE;
 
 EXHAUSTED:
@@ -489,7 +509,7 @@ Use ALLOCATE to expand the space.",
 		"Continues execution.",
 		1, vs_top[-1],Cnil,Cnil,Cnil);
 	vs_popp;
-	call_after_gbc_hook(t_cons);
+/* 	call_after_gbc_hook(t_cons); */
 	goto ONCE_MORE;
 #undef	tm
 }
@@ -504,7 +524,7 @@ object on_stack_cons(object x, object y)
 }
 
 
-DEFUN_NEW("ALLOCATED",object,fSallocated,SI,1,1,NONE,OO,OO,OO,OO,(object typ),"")
+DEFUNM_NEW("ALLOCATED",object,fSallocated,SI,1,1,NONE,OO,OO,OO,OO,(object typ),"")
 { struct typemanager *tm=(&tm_table[t_from_type(typ)]);
   tm = & tm_table[tm->tm_type];
   if (tm->tm_type == t_relocatable)
@@ -564,6 +584,11 @@ alloc_contblock(size_t n) {
 	g = FALSE;
 	n = ROUND_UP_PTR_CONT(n);
 
+	if ((p=maybe_alloc_on_stack(n,-1))) {
+/* 	  printf("alloc_contblock %u %d %u %p\n",getpid(),-1,n,p); */
+	  return p;
+	}
+
 ONCE_MORE:
 	CHECK_INTERRUPT;
 	for(cbpp= &cb_pointer; (*cbpp)!=NULL; cbpp= &(*cbpp)->cb_link)
@@ -608,7 +633,7 @@ ONCE_MORE:
 		  }
 		} 
 		g=j==maxcbpage ? TRUE : FALSE;
-		call_after_gbc_hook(t_contiguous);
+/* 		call_after_gbc_hook(t_contiguous); */
 		goto ONCE_MORE;
 EXHAUSTED:
 		vs_push(make_fixnum(ncbpage));
@@ -618,7 +643,7 @@ Use ALLOCATE-CONTIGUOUS-PAGES to expand the space.",
 			"Continues execution.", 1, vs_head,Cnil,Cnil,Cnil);
 		vs_popp;
 		g = FALSE;
-		call_after_gbc_hook(t_contiguous);
+/* 		call_after_gbc_hook(t_contiguous); */
 		goto ONCE_MORE;
 	}
       }
@@ -639,6 +664,7 @@ Use ALLOCATE-CONTIGUOUS-PAGES to expand the space.",
 	}
 	ncbpage += m;
 	insert_contblock(p+n, PAGESIZE*m - n);
+/* 	if (pending_agbch_call) call_after_gbc_hook(t_contiguous); */
 	return(p);
 }
 
@@ -727,6 +753,12 @@ alloc_relblock(size_t n) {
 	g = FALSE;
 	n = ROUND_UP_PTR(n);
 
+	if ((p=maybe_alloc_on_stack(n,-1))) {
+/* 	  printf("alloc_relblock %u %d %u %p\n",getpid(),-1,n,p); */
+	  return p;
+	}
+
+
 ONCE_MORE:
         CHECK_INTERRUPT;
 
@@ -768,16 +800,17 @@ ONCE_MORE:
 		goto EXHAUSTED;
 	      }
 	    } else if (nrbpage != i) {
-	      rb_end +=  (PAGESIZE* (nrbpage -i));
+/* 	      rb_end +=  (PAGESIZE* (nrbpage -i)); */
+	      rb_end =  heap_end+(holepage+nrbpage)*PAGESIZE;
 	      rb_limit = rb_end - 2*RB_GETA;
-	      if (page(rb_end) - page(heap_end) !=
-		  holepage + nrbpage)
-		FEerror("bad rb_end",0);
+/* 	      if (page(rb_end) - page(heap_end) != */
+/* 		  holepage + nrbpage) */
+/* 		FEerror("bad rb_end",0); */
 	      alloc_page(-( nrbpage + holepage));
 	    }
 	  } 
 	  g=i==nrbpage ? TRUE : FALSE;
-	  call_after_gbc_hook(t_relocatable);
+/* 	  call_after_gbc_hook(t_relocatable); */
 	  goto ONCE_MORE;
 EXHAUSTED:		
 	  if (rb_limit > rb_end - 2*RB_GETA)
@@ -790,7 +823,7 @@ Use ALLOCATE-RELOCATABLE-PAGES to expand the space.",
 		  "Continues execution.", 1, vs_head,Cnil,Cnil,Cnil);
 	  vs_popp;
 	  g = FALSE;
-	  call_after_gbc_hook(t_relocatable);
+/* 	  call_after_gbc_hook(t_relocatable); */
 	  goto ONCE_MORE;
 	}
 	p = rb_pointer;
@@ -961,6 +994,20 @@ gcl_init_alloc(void) {
   new_holepage = CORE_PAGES/HOLEDIV;
   nrbpage = CORE_PAGES/INIT_NRBDIV;
   
+#if 0    
+    fprintf ( stderr,
+              "gcl_init_alloc:"
+              " PAGESIZE %lx, MAXPAGE %lx,"
+              " MAXCORE %lx, CORE_PAGES %lx, INIT_NRBDIV %lx, INIT_HOLEDIV %lx, HOLEDIV %lx",
+              PAGESIZE,
+              MAXPAGE,
+              MAXCORE,
+              CORE_PAGES,
+              INIT_NRBDIV,
+              INIT_HOLEDIV,
+              HOLEDIV );
+#endif  
+
   alloc_page(-(holepage + nrbpage));
   
   rb_start = rb_pointer = heap_end + PAGESIZE*holepage;
@@ -983,6 +1030,7 @@ gcl_init_alloc(void) {
   init_tm(t_fixnum, "NFIXNUM",
 	  sizeof(struct fixnum_struct), 8192,20,0);
   init_tm(t_structure, "SSTRUCTURE", sizeof(struct structure), 5461,1,0 );
+  init_tm(t_ifun, "iIFUN", sizeof(struct ifun), 4096,1,0  );
   init_tm(t_cfun, "fCFUN", sizeof(struct cfun), 4096,1,0  );
   init_tm(t_sfun, "gSFUN", sizeof(struct sfun),409,1,0 );
   init_tm(t_string, "\"STRING", sizeof(struct string), 5461,1,0  );
@@ -1154,6 +1202,7 @@ DEFUN_NEW("ALLOCATE-CONTIGUOUS-PAGES",object,fSallocate_contiguous_pages,SI
 
   ncbpage += m;
   insert_contblock(p, PAGESIZE*m);
+/*   if (pending_agbch_call) call_after_gbc_hook(t_contiguous); */
   RETURN1(Ct);
 
 }
@@ -1200,6 +1249,7 @@ DEFUN_NEW("ALLOCATE-RELOCATABLE-PAGES",object,fSallocate_relocatable_pages,SI
   nrbpage = npages;
   rb_limit = rb_end - 2*RB_GETA;
   alloc_page(-(holepage + nrbpage));
+/*   if (pending_agbch_call) call_after_gbc_hook(t_relocatable); */
   vs_top = vs_base;
   vs_push(Ct);
   RETURN1(make_fixnum(npages));
@@ -1255,6 +1305,7 @@ DEFUN_NEW("ALLOCATE",object,fSallocate,SI
       for (;  tm->tm_npage < tm->tm_maxpage;  pp += PAGESIZE)
 	add_page_to_freelist(pp,tm);}
   
+/*   if (pending_agbch_call) call_after_gbc_hook(t); */
   RETURN1(Ct);
 
 }
@@ -1393,7 +1444,7 @@ DEFUN_NEW("GPROF-QUIT",object,fSgprof_quit,SI
 
 #endif
 
-DEFUN_NEW("SET-HOLE-SIZE",object,fSset_hole_size,SI
+DEFUNM_NEW("SET-HOLE-SIZE",object,fSset_hole_size,SI
        ,1,2,NONE,OI,IO,OO,OO,(fixnum npages,...),"")
 {
 
