@@ -61,22 +61,22 @@ ihs_function_name(object x)
 			return(sLlambda_closure);
 		if (y == sLlambda_block || y == sSlambda_block_expanded) {
 			x = x->c.c_cdr;
-			if (type_of(x) != t_cons)
+			if (!consp(x))
 				return(sLlambda_block);
 			return(x->c.c_car);
 		}
 		if (y == sLlambda_block_closure) {
 			x = x->c.c_cdr;
-			if (type_of(x) != t_cons)
+			if (!consp(x))
 				return(sLlambda_block_closure);
 			x = x->c.c_cdr;
-			if (type_of(x) != t_cons)
+			if (!consp(x))
 				return(sLlambda_block_closure);
 			x = x->c.c_cdr;
-			if (type_of(x) != t_cons)
+			if (!consp(x))
 				return(sLlambda_block_closure);
 			x = x->c.c_cdr;
-			if (type_of(x) != t_cons)
+			if (!consp(x))
 				return(sLlambda_block_closure);
 			return(x->c.c_car);
 		}
@@ -385,7 +385,7 @@ void
 FEinvalid_function(object obj)
 {
 #ifdef ANSI_COMMON_LISP
-    FEwrong_type_argument(obj,sLfunction);
+    TYPE_ERROR(obj,sLfunction);
 #else
     Icall_error_handler(sKinvalid_function,
 		     make_simple_string("~S is invalid as a function."),
@@ -614,7 +614,7 @@ DEFUN_NEW("SCH-FRS-BASE",object,fSsch_frs_base,SI
 	RETURN1(x0);
 }
 
-DEFUN_NEW("INTERNAL-SUPER-GO",object,fSinternal_super_go,SI
+DEFUNM_NEW("INTERNAL-SUPER-GO",object,fSinternal_super_go,SI
        ,3,3,NONE,OO,OO,OO,OO,(object tag,object x1,object x2),"")
 {
 	frame_ptr fr;
@@ -823,6 +823,7 @@ DEF_ORDINARY("UNDEFINED-FUNCTION",sKundefined_function,KEYWORD,"");
 DEF_ORDINARY("INVALID-FUNCTION",sKinvalid_function,KEYWORD,"");
 DEF_ORDINARY("PACKAGE-ERROR",sKpackage_error,KEYWORD,"");
 DEF_ORDINARY("PARSE-ERROR",sKparse_error,KEYWORD,"");
+DEF_ORDINARY("READER-ERROR",sKreader_error,KEYWORD,"");
 DEF_ORDINARY("CATCH",sKcatch,KEYWORD,"");
 DEF_ORDINARY("PROTECT",sKprotect,KEYWORD,"");
 DEF_ORDINARY("CATCHALL",sKcatchall,KEYWORD,"");
