@@ -227,13 +227,19 @@ funcall(object fun)
 
 	case t_ifun:
 	  {object x = fun->ifn.ifn_self;
-	  if (x) { fun = x; goto TOP;}
+	  if (x) { fun = x; break;}
 	  else
 	    FEundefined_function(fun);
 	  }
 	  
-	case t_cons:
-		break;
+ 	case t_cons:
+	  if (fun->c.c_car!=sLlambda &&
+	      fun->c.c_car!=sLlambda_closure &&
+	      fun->c.c_car!=sLlambda_block &&
+	      fun->c.c_car!=sSlambda_block_expanded &&
+	      fun->c.c_car!=sLlambda_block_closure)
+		FEinvalid_function(fun);
+	  break;
 
 	default:
 		FEinvalid_function(fun);
@@ -808,7 +814,7 @@ EVAL:
 	} else
 		eval1 = 0;
 
-	if (type_of(form) == t_cons)
+	if (consp(form))
 		goto APPLICATION;
 
 	if (type_of(form) != t_symbol)  RETURN1(form);
@@ -824,7 +830,7 @@ EVAL:
 
 	default:
 		/*  x = lex_var_sch(form);  */
-		for (x = lex_env[0];  type_of(x) == t_cons;  x = x->c.c_cdr)
+		for (x = lex_env[0];  consp(x);  x = x->c.c_cdr)
 			if (x->c.c_car->c.c_car == form) {
 				x = x->c.c_car->c.c_cdr;
 				if (endp(x))
@@ -869,7 +875,7 @@ APPLICATION:
 		return Ivs_values();
 	}
 	/*  x = lex_fd_sch(fun);  */
-	for (x = lex_env[1];  type_of(x) == t_cons;  x = x->c.c_cdr)
+	for (x = lex_env[1];  consp(x);  x = x->c.c_cdr)
 		if (x->c.c_car->c.c_car == fun) {
 			x = x->c.c_car;
 			if (MMcadr(x) == sLmacro) {
@@ -925,7 +931,7 @@ EVAL_ARGS:
 		 }
 
 LAMBDA:
-	if (type_of(fun) == t_cons && MMcar(fun) == sLlambda) {
+	if (consp(fun) && MMcar(fun) == sLlambda) {
 	  x = listA(4,sLlambda_closure,lex_env[0],lex_env[1],lex_env[2],Mcdr(fun));
 	  goto EVAL_ARGS;
 	}
@@ -978,7 +984,7 @@ EVAL:
 	} else
 		eval1 = 0;
 
-	if (type_of(form) == t_cons)
+	if (consp(form))
 		goto APPLICATION;
 
 	if (type_of(form) != t_symbol) {
@@ -1002,7 +1008,7 @@ EVAL:
 
 	default:
 		/*  x = lex_var_sch(form);  */
-		for (x = lex_env[0];  type_of(x) == t_cons;  x = x->c.c_cdr)
+		for (x = lex_env[0];  consp(x);  x = x->c.c_cdr)
 			if (x->c.c_car->c.c_car == form) {
 				x = x->c.c_car->c.c_cdr;
 				if (endp(x))
@@ -1051,7 +1057,7 @@ APPLICATION:
 		return;
 	}
 	/*  x = lex_fd_sch(fun);  */
-	for (x = lex_env[1];  type_of(x) == t_cons;  x = x->c.c_cdr)
+	for (x = lex_env[1];  consp(x);  x = x->c.c_cdr)
 		if (x->c.c_car->c.c_car == fun) {
 			x = x->c.c_car;
 			if (MMcadr(x) == sLmacro) {
@@ -1084,7 +1090,7 @@ EVAL_ARGS:
 	form = form->c.c_cdr;
 	base = vs_top;
 	top = vs_top;
-	while(!endp_prop(form)) {
+	while(!endp(form)) {
 		eval(MMcar(form));
 		top[0] = vs_base[0];
 		vs_top = ++top;
@@ -1106,7 +1112,7 @@ EVAL_ARGS:
 	return;
 
 LAMBDA:
-	if (type_of(fun) == t_cons && MMcar(fun) == sLlambda) {
+	if (consp(fun) && MMcar(fun) == sLlambda) {
 		temporary = make_cons(lex_env[2], fun->c.c_cdr);
 		temporary = make_cons(lex_env[1], temporary);
 		temporary = make_cons(lex_env[0], temporary);
@@ -1182,7 +1188,7 @@ DEFUNO_NEW("APPLY",object,fLapply,LISP
       }
 	
 
-DEFUNO_NEW("EVAL",object,fLeval,LISP
+DEFUNOM_NEW("EVAL",object,fLeval,LISP
        ,1,1,NONE,OO,OO,OO,OO,void,Leval,(object x0),"")
 {
 	object *lex = lex_env;
