@@ -218,7 +218,7 @@ mark_cons(object x) {
   
  BEGIN:  
   if (NULL_OR_ON_C_STACK(x->c.c_car)) goto MARK_CDR;
-  if (type_of(x->c.c_car) == t_cons) {
+  if (consp(x->c.c_car)) {
     if (is_marked_or_free(x->c.c_car))
       ;
     else {
@@ -231,7 +231,7 @@ mark_cons(object x) {
   if (NULL_OR_ON_C_STACK(x->c.c_cdr))
     return;
   x = Scdr(x);
-  if (type_of(x) == t_cons) {
+  if (consp(x)) {
     if (is_marked_or_free(x))
       return;
     mark(x);
@@ -408,6 +408,11 @@ mark_object(object x) {
     case aet_nnshort:
       j=sizeof(short)*x->a.a_dim;
       break;
+    case aet_int:
+    case aet_uint:
+    case aet_nnint:
+      j=sizeof(int)*x->a.a_dim;
+      break;
     default:
       j=sizeof(fixnum)*x->fixa.fixa_dim;}
     
@@ -531,7 +536,7 @@ mark_object(object x) {
     unsigned char * s_type = &SLOT_TYPE(def,0);
     unsigned short *s_pos= & SLOT_POS(def,0);
     for (i = 0, j = S_DATA(def)->length;  i < j;  i++)
-      if (s_type[i]==0) mark_object(STREF(object,x,s_pos[i]));
+      if (s_type[i]==aet_object) mark_object(STREF(object,x,s_pos[i]));
     if ((int)what_to_collect >= (int)t_contiguous) {
       if (inheap(x->str.str_self)) {
 	if (what_to_collect == t_contiguous)
@@ -1075,6 +1080,23 @@ char *old_rb_start;
 #define OPTIMIZE_MAX_PAGES (sSAoptimize_maximum_pagesA ==0 || sSAoptimize_maximum_pagesA->s.s_dbind !=sLnil) 
 #define IGNORE_MAX_PAGES (sSAignore_maximum_pagesA ==0 || sSAignore_maximum_pagesA->s.s_dbind !=sLnil) 
 
+DEFVAR("*AFTER-GBC-HOOK*",sSAafter_gbc_hookA,SI,sLnil,"");
+
+static struct symbol in_agbchs;
+static object in_agbch=(object)&in_agbchs;
+static void
+call_after_gbc_hook(t) { 
+
+  if (sSAafter_gbc_hookA && sSAafter_gbc_hookA->s.s_dbind!= Cnil) { 
+    bds_ptr old_bds_top = bds_top;
+    bds_bind(in_agbch,Ct);
+    set_up_string_register(tm_table[(int)t].tm_name+1);
+    ifuncall1(sSAafter_gbc_hookA->s.s_dbind,intern(string_register,system_package));
+    bds_unwind(old_bds_top);
+  }
+  
+}
+
 void
 GBC(enum type t) {
 
@@ -1086,6 +1108,9 @@ GBC(enum type t) {
 #ifdef DEBUG
   int tm=0;
 #endif
+
+  if (in_agbch->s.s_dbind==Ct)
+    return;
   
   mcsl=mcsh=&tm;
 
@@ -1420,6 +1445,7 @@ GBC(enum type t) {
   
   CHECK_INTERRUPT;
 
+  call_after_gbc_hook(t);
 
 }
 
@@ -1476,6 +1502,7 @@ FFN(siLroom_report)(void) {
   vs_push(make_fixnum(nrbpage));
   vs_push(make_fixnum(rbgbccount));
   for (i = 0;  i < (int)t_end;  i++) {
+    vs_check_push(make_simple_string(tm_table[i].tm_name+1));
     if (tm_table[i].tm_type == (enum type)i) {
       vs_check_push(make_fixnum(TM_NUSED(tm_table[i])));
       vs_push(make_fixnum(tm_table[i].tm_nfree+tm_table[i].tm_alt_nfree));
@@ -1485,7 +1512,7 @@ FFN(siLroom_report)(void) {
       vs_push(make_fixnum(tm_table[i].tm_size/sizeof(fixnum)));
     } else {
       vs_check_push(Cnil);
-      vs_push(make_fixnum(tm_table[i].tm_type));
+      vs_push(make_simple_string(tm_of(i)->tm_name+1));
       vs_push(Cnil);
       vs_push(Cnil);
       vs_push(Cnil);
