@@ -17,6 +17,10 @@ License for more details.
 */
 
 #include <string.h>
+#include <sys/types.h>
+#ifndef _WIN32
+#include <sys/wait.h>
+#endif
 
 #define IN_RUN_PROCESS
 #include "include.h"
@@ -358,7 +362,9 @@ int	server;
 	struct	hostent	*hp;
 	struct	sockaddr_in	sock_add;	/* Address of socket          */
 
+#ifndef STATIC_LINKING
 	if((hp = gethostbyname(host)) == NULL)
+#endif
 	{
 		FEerror("No such host.",0);
 	}
@@ -423,7 +429,7 @@ enum smmode smm;
 	stream->sm.sm_mode = (short)smm;
 	stream->sm.sm_fp = fp;
 
-	stream->sm.sm_object0 = sLstring_char;
+	stream->sm.sm_object0 = sLcharacter;
 	stream->sm.sm_object1 = host_l;
 	stream->sm.sm_int0 = stream->sm.sm_int1 = 0;
 	vs_push(stream);
@@ -579,9 +585,158 @@ FFN(siLmake_socket_pair)()
   make_socket_pair();
 }
 
+#define unpack_handle(a_,b_,c_) ({if (!consp(a_))\
+                                    TYPE_ERROR(a_,sLcons);\
+                                  if (type_of(a_->c.c_car)!=t_fixnum)\
+                                    TYPE_ERROR(a_->c.c_car,sLfixnum);\
+                                  b_=fix(a_->c.c_car);\
+                                  if (type_of(a_->c.c_cdr) != t_fixnum)\
+                                    TYPE_ERROR(a_->c.c_cdr,sLfixnum);\
+                                  c_=fix(a_->c.c_cdr);})
+
+
+DEFUNO_NEW("KILL",object,fSkill,SI,2,2,NONE,OO,IO,OO,OO,void,siLkill,(object x,fixnum err),"") {
+
+  fixnum k,l;
+  int e,status;
+
+  unpack_handle(x,k,l);
+
+  if (l>=0) {
+    ASSERT((e=waitpid(k,&status,WNOHANG))>=0);
+    if (e) {
+      if (!WIFEXITED(status)) {
+	ASSERT(WIFSIGNALED(status));
+	FEerror("Child %u died with signal %u\n",k,WTERMSIG(status));
+      } else if ((e=WEXITSTATUS(status)))
+	  FEerror("Child %u exited with error status %d\n",k,e);
+    } else {
+      ASSERT(!kill(k,SIGTERM));
+      ASSERT(waitpid(k,&status,0)==k);
+      if (WIFSIGNALED(status)) {
+	ASSERT(WTERMSIG(status)==SIGTERM);
+      } else {
+	ASSERT(WIFEXITED(status));
+	if ((e=WEXITSTATUS(status)))
+	  FEerror("Child %u exited with error status %d\n",k,e);
+      }
+    }
+/*     ASSERT(!close(l)); */
+    close(l);/*FIXME*/
+    x->c.c_cdr=make_fixnum(-1);
+  }
+  return Cnil;
+}
+  
+DEFUNO_NEW("SELECT-READ",object,fSselect_read,SI,2,2,NONE,IO,IO,OO,OO,void,siLselect_read,
+	   (object x,fixnum usec),"") {
+  fd_set fds;
+  fixnum max=-1,k,mask,i;
+  object y=x;
+  struct timeval tv={usec/1000000,usec%1000000};
+
+  FD_ZERO(&fds);
+  if (x!=Cnil && !consp(x))
+    TYPE_ERROR(x,sLlist);
+  for (;!endp(x);x=x->c.c_cdr) {
+    unpack_handle(x->c.c_car,i,k);
+    if (k<0) continue;/*closed stream*/
+    max=max<k ? k : max;
+    FD_SET(k,&fds);
+  }
+  select(max+1,&fds,NULL,NULL,usec < 0 ? NULL : &tv);
+  for (x=y,i=mask=0;!endp(x);x=x->c.c_cdr,i++) {
+    k=fix(x->c.c_car->c.c_cdr);
+    if (k<0) continue;
+    if (FD_ISSET(k,&fds))
+      mask|=(1<<i);
+  }
+  return (object)mask;
+}
+  
+DEFUNO_NEW("WRITE-POINTER-OBJECT",object,fSwrite_pointer_object,SI,2,2,NONE,OO,OO,OO,OO,
+	   void,siLwrite_pointer_object,(object x,object z),"") {
+
+  object y;
+  fixnum pid,s;
+
+  unpack_handle(z,pid,s);
+
+  y=x;
+  if (stack_alloc_end<=stack_alloc_start || !writable_ptr(x)) 
+    y=OBJNULL;
+  ASSERT(write(s,&y,sizeof(y))==sizeof(y)); 
+  if (y==OBJNULL) {
+    char b[BUFSIZ];
+    stack_alloc_off();
+    y=make_fd_stream(s,smm_output,"w",b);
+    prin1(x,y);
+    fclose(y->sm.sm_fp);
+  }
+  return Cnil;
+}
+
+DEFUNO_NEW("READ-POINTER-OBJECT",object,fSread_pointer_object,SI,1,1,NONE,OO,OO,OO,OO,
+	   void,siLread_pointer_object,(object z),"") {
+
+  object x;
+  fixnum pid,s;
+
+  unpack_handle(z,pid,s);
+
+  ASSERT(read(s,&x,sizeof(x))==sizeof(x));
+  if (x==OBJNULL) {
+    object y;
+    char b[BUFSIZ];
+    /*FIXME this could be somewhat faster if the malloc induced by
+      fdopen could be avoided.*/
+    y=make_fd_stream(s,smm_input,"r",b);
+    x=read_object(y);
+    fclose(y->sm.sm_fp);
+  }
+  FFN(fSkill)(z,1);
+  return x;
+
+}
+
+DEFVAR("*CHILD-STACK-ALLOC*",sSAchild_stack_allocA,SI,make_shortfloat(0.8),"");
+
+DEFUNO_NEW("FORK",object,fSfork,SI,0,0,NONE,OO,OO,OO,OO,void,siLfork,(void),"") {
+  int p[2],j=0;
+  pid_t pid;
+
+  ASSERT(!pipe(p));
+
+  ASSERT((pid=fork())>=0);
+  
+  if (!pid) {
+
+    j=1;
+    close(STDIN_FILENO);
+    close(STDOUT_FILENO);
+    close(STDERR_FILENO);
+      
+  }
+
+  close(p[1-j]);
+
+  return MMcons(make_fixnum(pid),make_fixnum(p[j]));
+
+}
+
+
+
 void
 gcl_init_socket_function()
-{
+{ 
+ 
+/*   struct sigaction sa; */
+/*   sa.sa_handler=SIG_IGN; */
+/*   sa.sa_flags=SA_NOCLDWAIT; */
+/*   sigemptyset(&sa.sa_mask); */
+  
+/*   sigaction(SIGCHLD,&sa,NULL); */
+  
   make_si_function("MAKE-SOCKET-STREAM", siLmake_socket_stream); 
   make_si_function("MAKE-SOCKET-PAIR", siLmake_socket_pair);
   make_si_function("RUN-PROCESS", siLrun_process);
