@@ -126,15 +126,6 @@ check_type_character(object *p)
 		*p = wrong_type_argument(sLcharacter, *p);
 }
 
-/* static void */
-/* check_type_string_char(object *p) */
-/* { */
-/* 	while (type_of(*p) != t_character || */
-/* 	       char_font((*p)) != 0 || */
-/* 	       char_bits((*p)) != 0) */
-/* 		*p = wrong_type_argument(sLcharacter, *p); */
-/* } */
-
 void
 check_type_symbol(object *p)
 {
@@ -190,7 +181,7 @@ check_type_string(object *p)
 void
 check_type_cons(object *p)
 {
-	while (type_of(*p) != t_cons)
+	while (!consp(*p))
 		*p = wrong_type_argument(sLcons, *p);
 }
 
@@ -202,13 +193,20 @@ check_type_stream(object *p)
 }
 
 /* Thankfully we can do this bit of non-lispy c stuff since we pass by reference. FIXME*/
+void
+check_type_readtable_no_default(object *p) {
+  
+  if (type_of(*p) != t_readtable)
+    *p = wrong_type_argument(sLreadtable, *p);
+
+}
+
 void
 check_type_readtable(object *p) {
   
   if (*p==Cnil)
     *p=standard_readtable;
-  if (type_of(*p) != t_readtable)
-    *p = wrong_type_argument(sLreadtable, *p);
+  check_type_readtable_no_default(p);
 
 }
 
@@ -476,6 +474,7 @@ DEF_ORDINARY("COMMON",sLcommon,LISP,"");
 DEF_ORDINARY("NULL",sLnull,LISP,"");
 DEF_ORDINARY("CONS",sLcons,LISP,"");
 DEF_ORDINARY("LIST",sLlist,LISP,"");
+DEF_ORDINARY("PROPER-LIST",siLproper_list,SI,"");
 DEF_ORDINARY("SYMBOL",sLsymbol,LISP,"");
 DEF_ORDINARY("ARRAY",sLarray,LISP,"");
 DEF_ORDINARY("VECTOR",sLvector,LISP,"");
@@ -495,7 +494,6 @@ DEF_ORDINARY("CHARACTER",sLcharacter,LISP,"");
 DEF_ORDINARY("NUMBER",sLnumber,LISP,"");
 DEF_ORDINARY("RATIONAL",sLrational,LISP,"");
 DEF_ORDINARY("FLOAT",sLfloat,LISP,"");
-DEF_ORDINARY("STRING-CHAR",sLstring_char,LISP,"");
 DEF_ORDINARY("INTEGER",sLinteger,LISP,"");
 DEF_ORDINARY("RATIO",sLratio,LISP,"");
 DEF_ORDINARY("SHORT-FLOAT",sLshort_float,LISP,"");
@@ -517,6 +515,11 @@ DEF_ORDINARY("NEGATIVE-SHORT",sLnegative_short,LISP,"");
 DEF_ORDINARY("SIGNED-SHORT",sLsigned_short,LISP,"");
 DEF_ORDINARY("UNSIGNED-SHORT",sLunsigned_short,LISP,"");
 
+DEF_ORDINARY("NON-NEGATIVE-INT",sLnon_negative_int,LISP,"");
+DEF_ORDINARY("NEGATIVE-INT",sLnegative_int,LISP,"");
+DEF_ORDINARY("SIGNED-INT",sLsigned_int,LISP,"");
+DEF_ORDINARY("UNSIGNED-INT",sLunsigned_int,LISP,"");
+
 DEF_ORDINARY("FIXNUM",sLfixnum,LISP,"");
 DEF_ORDINARY("NON-NEGATIVE-FIXNUM",sLnon_negative_fixnum,LISP,"");
 DEF_ORDINARY("NEGATIVE-FIXNUM",sLnegative_fixnum,LISP,"");
@@ -558,6 +561,12 @@ DEF_ORDINARY("UNSIGNED-SHORT",sLunsigned_short,LISP,"");
 DEF_ORDINARY("*",sLA,LISP,"");
 DEF_ORDINARY("PLUSP",sLplusp,LISP,"");
 DEF_ORDINARY("FILE-STREAM",sLfile_stream,LISP,"");
+DEF_ORDINARY("INPUT-STREAM",sLinput_stream,SI,"");
+DEF_ORDINARY("OUTPUT-STREAM",sLoutput_stream,SI,"");
+DEF_ORDINARY("PROCLAIMED-ARG-TYPES",sSproclaimed_arg_types,SI,"");
+DEF_ORDINARY("PROCLAIMED-RETURN-TYPE",sSproclaimed_return_type,SI,"");
+DEF_ORDINARY("PROCLAIMED-FUNCTION",sSproclaimed_function,SI,"");
+
 
 /* logical pathnames exist even in non ansi gcl */
 DEF_ORDINARY("LOGICAL-PATHNAME",sLlogical_pathname,LISP,"");
@@ -619,10 +628,16 @@ DEF_ORDINARY("WARNING",sLwarning,LISP,"");
 DEF_ORDINARY("DYNAMIC-EXTENT",sLdynamic_extent,LISP,"");
 #endif
 
-DEFCONST("CHAR-LENGTH",sSchar_length,SI,small_fixnum(CHAR_SIZE),"Size in bits of a character");
-DEFCONST("SHORT-LENGTH",sSshort_length,SI,small_fixnum(CHAR_SIZE*sizeof(short)),"Size in bits of a short integer");
-DEFCONST("FIXNUM-LENGTH",sSfixnum_length,SI,small_fixnum(CHAR_SIZE*sizeof(fixnum)),"Size in bits of a short integer");
-DEFCONST("LFIXNUM-LENGTH",sSlfixnum_length,SI,small_fixnum(CHAR_SIZE*sizeof(lfixnum)),"Size in bits of a short integer");
+DEFCONST("CHAR-LENGTH",   sSchar_length,   SI,small_fixnum(CHAR_SIZE),
+	 "Size in bits of a character");
+DEFCONST("SHORT-LENGTH",  sSshort_length,  SI,small_fixnum(CHAR_SIZE*sizeof(short)),
+	 "Size in bits of a short integer");
+DEFCONST("INT-LENGTH", sSint_length, SI,small_fixnum(CHAR_SIZE*sizeof(int)),
+	 "Size in bits of an int");
+DEFCONST("FIXNUM-LENGTH", sSfixnum_length, SI,small_fixnum(CHAR_SIZE*sizeof(fixnum)),
+	 "Size in bits of a fixnum");
+DEFCONST("LFIXNUM-LENGTH",sSlfixnum_length,SI,small_fixnum(CHAR_SIZE*sizeof(lfixnum)),
+	 "Size in bits of a long fixnum");
 
 void     
 gcl_init_typespec(void)
