@@ -124,7 +124,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #endif
 #include <mach-o/nlist.h>
 #include <mach-o/getsect.h>
-#include <objc/malloc.h>
+#include <malloc/malloc.h>
 
 #include <sys/mman.h>
 
@@ -1274,7 +1274,7 @@ void init_darwin_zone_compat ()
 
 /* Replacement for broken sbrk(2).  */
 
-char *sbrk_macosx (int incr)
+void *my_sbrk (int incr)
 {
   char               *temp, *ptr;
   kern_return_t       rtn;
